#!/usr/bin/env python
# Copyright (c) Facebook, Inc. and its affiliates.
'''
./export_model.py --config-file /frankmocap-main/detectors/hand_only_detector/faster_rcnn_X_101_32x8d_FPN_3x_100DOH.yaml \
    --output ./output --export-method tracing --format onnx \
    MODEL.WEIGHTS /frankmocap-main/extra_data/hand_module/hand_detector/model_0529999.pth \
    MODEL.DEVICE cuda
'''
import argparse
import os
from typing import Dict, List, Tuple
import torch
from torch import Tensor, nn

import detectron2.data.transforms as T
from detectron2.checkpoint import DetectionCheckpointer
from detectron2.config import get_cfg
from detectron2.data import build_detection_test_loader, detection_utils
from detectron2.evaluation import COCOEvaluator, inference_on_dataset, print_csv_format
from detectron2.export import TracingAdapter, dump_torchscript_IR, scripting_with_instances
from detectron2.modeling import GeneralizedRCNN, RetinaNet, build_model
from detectron2.modeling.postprocessing import detector_postprocess
from detectron2.projects.point_rend import add_pointrend_config
from detectron2.structures import Boxes
from detectron2.utils.env import TORCH_VERSION
from detectron2.utils.file_io import PathManager
from detectron2.utils.logger import setup_logger


def setup_cfg(args):
    cfg = get_cfg()
    # cuda context is initialized before creating dataloader, so we don't fork anymore
    cfg.DATALOADER.NUM_WORKERS = 0
    add_pointrend_config(cfg)
    cfg.merge_from_file(args.config_file)
    cfg.merge_from_list(args.opts)
    cfg.freeze()
    return cfg


def export_caffe2_tracing(cfg, torch_model, inputs):
    from detectron2.export import Caffe2Tracer

    tracer = Caffe2Tracer(cfg, torch_model, inputs)
    if args.format == "caffe2":
        caffe2_model = tracer.export_caffe2()
        caffe2_model.save_protobuf(args.output)
        # draw the caffe2 graph
        caffe2_model.save_graph(os.path.join(args.output, "model.svg"), inputs=inputs)
        return caffe2_model
    elif args.format == "onnx":
        import onnx

        onnx_model = tracer.export_onnx()
        onnx.save(onnx_model, os.path.join(args.output, "model.onnx"))
    elif args.format == "torchscript":
        ts_model = tracer.export_torchscript()
        with PathManager.open(os.path.join(args.output, "model.ts"), "wb") as f:
            torch.jit.save(ts_model, f)
        dump_torchscript_IR(ts_model, args.output)


# experimental. API not yet final
def export_scripting(torch_model):
    assert TORCH_VERSION >= (1, 8)
    fields = {
        "proposal_boxes": Boxes,
        "objectness_logits": Tensor,
        "pred_boxes": Boxes,
        "scores": Tensor,
        "pred_classes": Tensor,
        "pred_masks": Tensor,
        "pred_keypoints": torch.Tensor,
        "pred_keypoint_heatmaps": torch.Tensor,
    }
    assert args.format == "torchscript", "Scripting only supports torchscript format."

    class ScriptableAdapterBase(nn.Module):
        # Use this adapter to workaround https://github.com/pytorch/pytorch/issues/46944
        # by not retuning instances but dicts. Otherwise the exported model is not deployable
        def __init__(self):
            super().__init__()
            self.model = torch_model
            self.eval()

    if isinstance(torch_model, GeneralizedRCNN):

        class ScriptableAdapter(ScriptableAdapterBase):
            def forward(self, inputs: Tuple[Dict[str, torch.Tensor]]) -> List[Dict[str, Tensor]]:
                instances = self.model.inference(inputs, do_postprocess=False)
                return [i.get_fields() for i in instances]

    else:

        class ScriptableAdapter(ScriptableAdapterBase):
            def forward(self, inputs: Tuple[Dict[str, torch.Tensor]]) -> List[Dict[str, Tensor]]:
                instances = self.model(inputs)
                return [i.get_fields() for i in instances]

    ts_model = scripting_with_instances(ScriptableAdapter(), fields)
    with PathManager.open(os.path.join(args.output, "model.ts"), "wb") as f:
        torch.jit.save(ts_model, f)
    dump_torchscript_IR(ts_model, args.output)
    return None


# experimental. API not yet final
def export_tracing(torch_model, inputs):
    assert TORCH_VERSION >= (1, 8)
    image = inputs[0]["image"]
    inputs = [{"image": image}]  # remove other unused keys

    if isinstance(torch_model, GeneralizedRCNN):

        def inference(model, inputs):
            # use do_postprocess=False so it returns ROI mask
            inst = model.inference(inputs, do_postprocess=False)[0]
            return [{"instances": inst}]

    else:
        inference = None  # assume that we just call the model directly

    traceable_model = TracingAdapter(torch_model, inputs, inference)

    if args.format == "torchscript":
        ts_model = torch.jit.trace(traceable_model, (image,))
        with PathManager.open(os.path.join(args.output, "model.ts"), "wb") as f:
            torch.jit.save(ts_model, f)
        dump_torchscript_IR(ts_model, args.output)
    elif args.format == "onnx":
        with PathManager.open(os.path.join(args.output, "model.onnx"), "wb") as f:
            torch.onnx.export(traceable_model, (image,), f, opset_version=11)
    logger.info("Inputs schema: " + str(traceable_model.inputs_schema))
    logger.info("Outputs schema: " + str(traceable_model.outputs_schema))

    if args.format != "torchscript":
        return None
    if not isinstance(torch_model, (GeneralizedRCNN, RetinaNet)):
        return None

    def eval_wrapper(inputs):
        """
        The exported model does not contain the final resize step, which is typically
        unused in deployment but needed for evaluation. We add it manually here.
        """
        input = inputs[0]
        instances = traceable_model.outputs_schema(ts_model(input["image"]))[0]["instances"]
        postprocessed = detector_postprocess(instances, input["height"], input["width"])
        return [{"instances": postprocessed}]

    return eval_wrapper


def get_sample_inputs(args):
    if args.sample_image is None:
        # get a first batch from dataset
        data_loader = build_detection_test_loader(cfg, cfg.DATASETS.TEST[0])
        first_batch = next(iter(data_loader))
        return first_batch
    else:
        # get a sample data
        original_image = detection_utils.read_image(args.sample_image, format=cfg.INPUT.FORMAT)
        # Do same preprocessing as DefaultPredictor
        aug = T.ResizeShortestEdge(
            [cfg.INPUT.MIN_SIZE_TEST, cfg.INPUT.MIN_SIZE_TEST], cfg.INPUT.MAX_SIZE_TEST
        )
        height, width = original_image.shape[:2]
        image = aug.get_transform(original_image).apply_image(original_image)
        image = torch.as_tensor(image.astype("float32").transpose(2, 0, 1))

        inputs = {"image": image, "height": height, "width": width}

        # Sample ready
        sample_inputs = [inputs]
        return sample_inputs


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Export a model for deployment.")
    parser.add_argument(
        "--format",
        choices=["caffe2", "onnx", "torchscript"],
        help="output format",
        default="torchscript",
    )
    parser.add_argument(
        "--export-method",
        choices=["caffe2_tracing", "tracing", "scripting"],
        help="Method to export models",
        default="tracing",
    )
    parser.add_argument("--config-file", default="", metavar="FILE", help="path to config file")
    parser.add_argument("--sample-image", default=None, type=str, help="sample image for input")
    parser.add_argument("--run-eval", action="store_true")
    parser.add_argument("--output", help="output directory for the converted model")
    parser.add_argument(
        "opts",
        help="Modify config options using the command-line",
        default=None,
        nargs=argparse.REMAINDER,
    )
    args = parser.parse_args()
    logger = setup_logger()
    logger.info("Command line arguments: " + str(args))
    PathManager.mkdirs(args.output)
    # Disable respecialization on new shapes. Otherwise --run-eval will be slow
    torch._C._jit_set_bailout_depth(1)

    cfg = setup_cfg(args)

    # create a torch model
    torch_model = build_model(cfg)
    DetectionCheckpointer(torch_model).resume_or_load(cfg.MODEL.WEIGHTS)
    torch_model.eval()

    # get sample data
    sample_inputs = get_sample_inputs(args)

    # convert and save model
    if args.export_method == "caffe2_tracing":
        exported_model = export_caffe2_tracing(cfg, torch_model, sample_inputs)
    elif args.export_method == "scripting":
        exported_model = export_scripting(torch_model)
    elif args.export_method == "tracing":
        exported_model = export_tracing(torch_model, sample_inputs)

    # run evaluation with the converted model
    if args.run_eval:
        assert exported_model is not None, (
            "Python inference is not yet implemented for "
            f"export_method={args.export_method}, format={args.format}."
        )
        logger.info("Running evaluation ... this takes a long time if you export to CPU.")
        dataset = cfg.DATASETS.TEST[0]
        data_loader = build_detection_test_loader(cfg, dataset)
        # NOTE: hard-coded evaluator. change to the evaluator for your dataset
        evaluator = COCOEvaluator(dataset, output_dir=args.output)
        metrics = inference_on_dataset(exported_model, data_loader, evaluator)
        print_csv_format(metrics)



import os
from pathlib import Path
from subprocess import Popen
import sys
import time
import os

'''
env config
'''

trt_exec_dict = {
    8003: './TensorRT-8.0.0.3/targets/x86_64-linux-gnu/bin/trtexec'
}
LD_LIBRARY_PATH = {
    8003: './TensorRT-8.0.0.3/targets/x86_64-linux-gnu/lib'
}

'''env config'''
trt_exec_version = 8003
os.environ['LD_LIBRARY_PATH'] = LD_LIBRARY_PATH[trt_exec_version]

if __name__ == '__main__':

    '''config'''
    procesing_onnx_name = 'test.onnx'
    batch_size = 1
    latent_size = 512
    input_size = 256
    '''simple onnx'''
    input_shape = "target:%d,3,%d,%d vsid:%d,%d" % (
    abs(batch_size), input_size, input_size, abs(batch_size), latent_size)
    print('start simplifier onnx')
    processing_simple_onnx_name = procesing_onnx_name.split('.')[
                                      0] + '_sim' + '_bs%d' % batch_size + '_%d.onnx' % trt_exec_version
    if batch_size == -1:
        pb_sim_clt = 'python -m onnxsim  %s  %s  --dynamic-input-shape --input-shape %s ' \
                     % (procesing_onnx_name, processing_simple_onnx_name, input_shape)
    else:
        pb_sim_clt = 'python -m onnxsim  %s  %s --input-shape %s ' \
                     % (procesing_onnx_name, processing_simple_onnx_name, input_shape)

    retcode = Popen(pb_sim_clt, shell=True).wait()

    '''
    sim onnx 2 trt
    '''
    fmt = 16
    workspace = 512 * 20
    iofmt = 16
    print('start convert trt')
    processing_trt_name = processing_simple_onnx_name.split('.')[0] + '_fp%d_ws%d_io%d' % (
    fmt, workspace, iofmt) + '.trt'
    if batch_size == -1:
        if fmt == 16:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s  --fp%d --workspace=%d --minShapes=target:1x3x256x256,vsid:1x512 --optShapes=target:1x3x256x256,vsid:1x512 --maxShapes=target:1x3x256x256,vsid:1x512' \
                         % (
                         trt_exec_dict[trt_exec_version], processing_simple_onnx_name, processing_trt_name, fmt, 8096)
        else:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s   --workspace=%d --minShapes=target:1x3x256x256,vsid:1x512 --optShapes=target:8x3x256x256,vsid:8x512 --maxShapes=target:32x3x256x256,vsid:32x512' \
                         % (trt_exec_dict[trt_exec_version], processing_simple_onnx_name, processing_trt_name, 8096)
    else:
        if fmt == 16:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s --explicitBatch --fp%d --workspace=%d' \
                         % (trt_exec_dict[trt_exec_version], processing_simple_onnx_name, processing_trt_name, fmt,
                            workspace * batch_size)
        else:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s --explicitBatch  --workspace=%d' \
                         % (trt_exec_dict[trt_exec_version], processing_simple_onnx_name, processing_trt_name,
                            workspace * batch_size)
    if iofmt != 32:
        pb_trt_clt += ' --inputIOFormats=fp16:chw --outputIOFormats=fp16:chw'
    retcode = Popen(pb_trt_clt, shell=True).wait()



# Copyright (c) OpenMMLab. All rights reserved.
from typing import Dict, Sequence, Union
import ctypes
from cv2box import try_import

trt = try_import('tensorrt', 'trt_wrapper_mmdeploy need tensorrt')
torch = try_import('torch', 'trt_wrapper_mmdeploy need torch')


def torch_dtype_from_trt(dtype: trt.DataType) -> torch.dtype:
    """Convert pytorch dtype to TensorRT dtype.

    Args:
        dtype (str.DataType): The data type in tensorrt.

    Returns:
        torch.dtype: The corresponding data type in torch.
    """

    if dtype == trt.bool:
        return torch.bool
    elif dtype == trt.int8:
        return torch.int8
    elif dtype == trt.int32:
        return torch.int32
    elif dtype == trt.float16:
        return torch.float16
    elif dtype == trt.float32:
        return torch.float32
    else:
        raise TypeError(f'{dtype} is not supported by torch')


def torch_device_from_trt(device: trt.TensorLocation):
    """Convert pytorch device to TensorRT device.

    Args:
        device (trt.TensorLocation): The device in tensorrt.
    Returns:
        torch.device: The corresponding device in torch.
    """
    if device == trt.TensorLocation.DEVICE:
        return torch.device('cuda')
    elif device == trt.TensorLocation.HOST:
        return torch.device('cpu')
    else:
        return TypeError(f'{device} is not supported by torch')


class TRTWrapper:

    def __init__(self,
                 engine: Union[str, trt.ICudaEngine], ):
        ctypes.CDLL('pretrain_models/trt_lib/libmmdeploy_tensorrt_ops.so')
        # trt.init_libnvinfer_plugins(trt.Logger(trt.Logger.WARNING), "")
        with trt.Logger() as logger, trt.Runtime(logger) as runtime:
            with open(engine, mode='rb') as f:
                engine_bytes = f.read()
            self.engine = runtime.deserialize_cuda_engine(engine_bytes)

        if not isinstance(self.engine, trt.ICudaEngine):
            raise TypeError(f'`engine` should be str or trt.ICudaEngine, \
                but given: {type(self.engine)}')

        self.context = self.engine.create_execution_context()

        self.__load_io_names()

    def __load_io_names(self):
        """Load input/output names from engine."""
        names = [_ for _ in self.engine]
        input_names = list(filter(self.engine.binding_is_input, names))
        # print(input_names)
        self._input_names = input_names

        output_names = list(set(names) - set(input_names))
        self._output_names = output_names

    def forward(self, inputs: Dict[str,
                                   torch.Tensor]) -> Dict[str, torch.Tensor]:
        """Run forward inference.

        Args:
            inputs (Dict[str, torch.Tensor]): The input name and tensor pairs.

        Return:
            Dict[str, torch.Tensor]: The output name and tensor pairs.
        """
        assert self._input_names is not None
        assert self._output_names is not None
        bindings = [None] * (len(self._input_names) + len(self._output_names))

        profile_id = 0
        for input_name, input_tensor in inputs.items():
            # check if input shape is valid
            profile = self.engine.get_profile_shape(profile_id, input_name)
            assert input_tensor.dim() == len(
                profile[0]), 'Input dim is different from engine profile.'
            for s_min, s_input, s_max in zip(profile[0], input_tensor.shape,
                                             profile[2]):
                assert s_min <= s_input <= s_max, \
                    'Input shape should be between ' \
                    + f'{profile[0]} and {profile[2]}' \
                    + f' but get {tuple(input_tensor.shape)}.'
            idx = self.engine.get_binding_index(input_name)

            # All input tensors must be gpu variables
            assert 'cuda' in input_tensor.device.type
            input_tensor = input_tensor.contiguous()
            if input_tensor.dtype == torch.long:
                input_tensor = input_tensor.int()
            self.context.set_binding_shape(idx, tuple(input_tensor.shape))
            bindings[idx] = input_tensor.contiguous().data_ptr()

        # create output tensors
        outputs = {}
        for output_name in self._output_names:
            idx = self.engine.get_binding_index(output_name)
            dtype = torch_dtype_from_trt(self.engine.get_binding_dtype(idx))
            shape = tuple(self.context.get_binding_shape(idx))

            device = torch_device_from_trt(self.engine.get_location(idx))
            output = torch.empty(size=shape, dtype=dtype, device=device)
            outputs[output_name] = output
            bindings[idx] = output.data_ptr()

        self.__trt_execute(bindings=bindings)

        return outputs

    def __trt_execute(self, bindings: Sequence[int]):
        """Run inference with TensorRT.

        Args:
            bindings (list[int]): A list of integer binding the input/output.
        """
        self.context.execute_async_v2(bindings,
                                      torch.cuda.current_stream().cuda_stream)



# -- coding: utf-8 --
# @Time : 2022/8/26
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
from .bbox_detector_mmdet import BboxDetectorBase
from .kp_detector_mmpose import KpDetectorBase



import os
from pathlib import Path
from subprocess import Popen
import sys
import time
import os

'''
env config
'''

trt_exec_dict = {
    7234: './TensorRT-7.2.3.4.Ubuntu-18.04.x86_64-gnu.cuda-11.1.cudnn8.1/TensorRT-7.2.3.4/targets/x86_64-linux-gnu/bin/trtexec',
    8003: './TensorRT-8.0.0.3/targets/x86_64-linux-gnu/bin/trtexec',
    8206: './TensorRT-8.2.0.6/targets/x86_64-linux-gnu/bin/trtexec'
}
LD_LIBRARY_PATH = {
    7234: './TensorRT-7.2.3.4.Ubuntu-18.04.x86_64-gnu.cuda-11.1.cudnn8.1/TensorRT-7.2.3.4/targets/x86_64-linux-gnu/lib',
    8003: './TensorRT-8.0.0.3/targets/x86_64-linux-gnu/lib',
    8206: './TensorRT-8.2.0.6/targets/x86_64-linux-gnu/lib'
}

'''env config'''
trt_exec_version = 8003
os.environ['LD_LIBRARY_PATH'] = LD_LIBRARY_PATH[trt_exec_version]

if __name__ == '__main__':

    '''config'''
    processing_onnx_name = '9A_64k_sim_bs4_8003.onnx'
    # processing_onnx_name='9A_64k.onnx'

    batch_size = 8
    latent_size = 512
    input_size = 256

    '''sim onnx 2 trt'''
    fmt = 16
    workspace = 512 * 80
    iofmt = 16
    print('start convert trt')
    processing_trt_name = processing_onnx_name.split('.')[0] + '_fp%d_ws%d_io%d' % (fmt, workspace, iofmt) + '.trt'
    if batch_size == -1:
        if fmt == 16:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s  --fp%d --workspace=%d --minShapes=target:1x3x256x256,vsid:1x512 --optShapes=target:1x3x256x256,vsid:1x512 --maxShapes=target:2x3x256x256,vsid:2x512' \
                         % (trt_exec_dict[trt_exec_version], processing_onnx_name, processing_trt_name, fmt,
                            abs(workspace * batch_size))
        else:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s   --workspace=%d --minShapes=target:1x3x256x256,vsid:1x512 --optShapes=target:8x3x256x256,vsid:8x512 --maxShapes=target:32x3x256x256,vsid:32x512' \
                         % (trt_exec_dict[trt_exec_version], processing_onnx_name, processing_trt_name,
                            abs(workspace * batch_size))
    else:
        if fmt == 16:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s --explicitBatch --fp%d --workspace=%d' \
                         % (trt_exec_dict[trt_exec_version], processing_onnx_name, processing_trt_name, fmt, workspace)
        else:
            pb_trt_clt = '%s --onnx=%s --saveEngine=%s --explicitBatch  --workspace=%d' \
                         % (trt_exec_dict[trt_exec_version], processing_onnx_name, processing_trt_name, workspace)
    if iofmt != 32:
        pb_trt_clt += ' --inputIOFormats=fp16:chw --outputIOFormats=fp16:chw'
    retcode = Popen(pb_trt_clt, shell=True).wait()



# -- coding: utf-8 --
# @Time : 2021/12/15
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
import os
import cv2
import time
import numpy as np
import tensorrt as trt
import pycuda.driver as cuda
import pycuda.autoinit
import torch


class TRTWrapperSelf:
    def __init__(self, trt_model_path):

        self.trt_model_path = trt_model_path

        # import pycuda.driver as cuda
        # import pycuda.autoinit
        self.cfx = cuda.Device(0).make_context()
        trt.init_libnvinfer_plugins(trt.Logger(trt.Logger.WARNING), "")
        runtime = trt.Runtime(trt.Logger(trt.Logger.WARNING))
        # f = load_encrypt_model(MODEL_P)

        # device_name = cuda.Device(0).name()

        f = open(self.trt_model_path, "rb")
        self.engine = runtime.deserialize_cuda_engine(f.read())
        # self.engine = runtime.deserialize_cuda_engine(load_encrypt_model(model_p))
        self.context = self.engine.create_execution_context()
        self.stream = cuda.Stream()
        self.host_inputs = []
        self.host_outputs = []
        self.host_size = 10
        for i in range(self.host_size):
            self.host_outputs.append([])

        self.cuda_inputs = []

        self.cuda_outputs = []
        self.bindings = []
        self.shape_outputs = []
        self.nbytes_list = []

        # # windows下需要写死，不然报错
        # size_list = [331776, 919296]# [196608, 512, 65536, 196608]

        idx = 0
        for binding in self.engine:
            shape = self.engine.get_binding_shape(binding)
            # size = size_list[idx]
            size = trt.volume(shape) * self.engine.max_batch_size
            host_mem = cuda.pagelocked_empty(size, np.float16)
            cuda_mem = cuda.mem_alloc(host_mem.nbytes)
            self.bindings.append(int(cuda_mem))
            if self.engine.binding_is_input(binding):
                self.host_inputs.append(host_mem)
                self.cuda_inputs.append(cuda_mem)
            else:
                for i in range(self.host_size):
                    self.host_outputs[i].append(host_mem)
                self.cuda_outputs.append(cuda_mem)
                self.shape_outputs.append(shape)
                self.nbytes_list.append(host_mem.nbytes)
            idx += 1
            if idx == 4:
                break
        self.host_idx = 0
        # 将init阶段的context pop掉，不然退出时pycuda会报错
        self.cfx.pop()

    def forward(self, input):  # result gets copied into output
        self.cfx.push()
        # Transfer input data to device
        # import pycuda.driver as cuda
        # import pycuda.autoinit
        for i in range(len(self.cuda_inputs)):
            np.copyto(self.host_inputs[i], input[i].ravel())
            cuda.memcpy_htod_async(self.cuda_inputs[i], self.host_inputs[i], self.stream)
        # Execute model
        self.context.execute_async_v2(self.bindings, self.stream.handle, None)
        # Transfer predictions back
        output = []
        for i in range(len(self.cuda_outputs)):
            cuda.memcpy_dtoh_async(self.host_outputs[self.host_idx][i], self.cuda_outputs[i], self.stream)
            output.append(self.host_outputs[self.host_idx][i].reshape((-1,) + self.shape_outputs[i][1:]))
        self.host_idx = 0 if self.host_idx + 1 >= self.host_size else self.host_idx + 1
        # Syncronize threads
        self.stream.synchronize()
        self.cfx.pop()
        return output

if __name__ == '__main__':
    inputs = torch.randn(1, 3, 384, 288)
    model = TRTWrapperSelf(
        '/pretrain_models/mocap_lib/coco_whole_body/hrnet_w48_coco_wholebody_384x288_dark-f5726563_20200918.engine')
    print(model.forward(inputs.to(dtype=torch.float16)))


# -- coding: utf-8 --
# @Time : 2023/2/21
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
"""
ref https://github.com/PINTO0309/snc4onnx

pip install -U onnx \
&& pip install -U onnx-simplifier \
&& python3 -m pip install -U onnx_graphsurgeon --index-url https://pypi.ngc.nvidia.com \
&& pip install -U snc4onnx
"""

# from snc4onnx import combine
#
# combined_graph = combine(
#     srcop_destop=[
#         ['output', 'flow_init']
#     ],
#     op_prefixes_after_merging=[
#         'init',
#         'next',
#     ],
#     input_onnx_file_paths=[
#         'crestereo_init_iter2_120x160.onnx',
#         'crestereo_next_iter2_240x320.onnx',
#     ],
#     non_verbose=True,
# )

"""
ref https://github.com/onnx/onnx/blob/main/docs/PythonAPIOverview.md#onnx-compose
"""
import onnx

model1 = onnx.load("path/to/model1.onnx")
new_model1 = onnx.compose.add_prefix(model1, prefix="m1/")

model2 = onnx.load("path/to/model2.onnx")
new_model2 = onnx.compose.add_prefix(model2, prefix="m2/")

combined_model = onnx.compose.merge_models(
    new_model1, new_model2,
    io_map=[
        ("m1/C", "m2/X"),
        ("m1/D", "m2/Y")
    ]
)



# -- coding: utf-8 --
# @Time : 2021/11/29
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone

import onnxruntime
import numpy as np
from cv2box import MyFpsCounter


def init_session(onnx_path, provider='gpu'):
    if provider == 'gpu':
        providers = (
            "CUDAExecutionProvider",
            {'device_id': 0, }
        )
    elif provider == 'trt':
        providers = (
            'TensorrtExecutionProvider',
            {'trt_engine_cache_enable': True, 'trt_fp16_enable': False, }
        )
    elif provider == 'trt16':
        providers = (
            'TensorrtExecutionProvider',
            {'trt_engine_cache_enable': True, 'trt_fp16_enable': True, }
        )
    elif provider == 'trt8':
        providers = (
            'TensorrtExecutionProvider',
            {'trt_engine_cache_enable': True, 'trt_int8_enable': True, }
        )
    else:
        providers = "CPUExecutionProvider"

    # onnxruntime.set_default_logger_severity(3)
    session_options = onnxruntime.SessionOptions()
    session_options.log_severity_level = 3
    onnx_session = onnxruntime.InferenceSession(onnx_path, session_options, providers=[providers])
    return onnx_session


class OnnxModelPickable:  # This is a wrapper to make the current InferenceSession class pickable.
    def __init__(self, onnx_path, provider='gpu', input_dynamic_shape=None):
        self.onnx_path = onnx_path
        self.provider = provider
        # self.onnx_session = init_session(self.onnx_path, self.provider)

    def get_input_feed(self, input_name, image_tensor):
        """
        input_feed={self.input_name: image_tensor}
        :param input_name:
        :param image_tensor:
        :return:
        """
        input_feed = {}
        for name in input_name:
            input_feed[name] = image_tensor
        return input_feed

    def get_output_info(self, onnx_session):
        """
        output_name = onnx_session.get_outputs()[0].name
        :param onnx_session:
        :return:
        """
        output_name = []
        output_shape = []
        for node in onnx_session.get_outputs():
            output_name.append(node.name)
            output_shape.append(node.shape)
        return output_name, output_shape

    def get_input_info(self, onnx_session):
        """
        input_name = onnx_session.get_inputs()[0].name
        :param onnx_session:
        :return:
        """
        input_name = []
        input_shape = []
        for node in onnx_session.get_inputs():
            input_name.append(node.name)
            input_shape.append(node.shape)
        return input_name, input_shape

    def forward(self, image_tensor, trans=False):
        '''
        image_tensor = image.transpose(2, 0, 1)
        image_tensor = image_tensor[np.newaxis, :]
        onnx_session.run([output_name], {input_name: x})
        :param image_tensor:
        :return:
        '''
        # 输入数据的类型必须与模型一致,以下三种写法都是可以的
        # scores, boxes = self.onnx_session.run(None, {self.input_name: image_tensor})
        # scores, boxes = self.onnx_session.run(self.output_name, input_feed={self.input_name: image_tensor})
        if trans:
            image_tensor = image_tensor.transpose(2, 0, 1)
            image_tensor = image_tensor[np.newaxis, :]
        image_tensor = np.ascontiguousarray(image_tensor)
        input_name, _ = self.get_input_info(self.onnx_session)
        output_name, _ = self.get_output_info(self.onnx_session)
        input_feed = self.get_input_feed(input_name, image_tensor)
        return self.onnx_session.run(output_name, input_feed=input_feed)

    def __getstate__(self):
        return {
            'onnx_path': self.onnx_path,
            'provider': self.provider,
        }

    def __setstate__(self, values):
        self.onnx_path = values['onnx_path']
        self.provider = values['provider']
        self.onnx_session = init_session(self.onnx_path, self.provider)



# -- coding: utf-8 --
# @Time : 2022/10/9
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
import torch


class TJMWrapper:
    def __init__(self, tjm_path, provider='gpu'):
        self.provider = provider
        self.model = torch.jit.load(tjm_path)
        self.model.eval()

        if self.provider == 'cpu':
            pass
        else:
            assert (torch.cuda.is_available())
            if self.provider == 'gpu':
                gpu_ids = [0]
            else:
                gpu_ids = self.provider
            self.model.to(gpu_ids[0])
            self.facenet = torch.nn.DataParallel(self.model, gpu_ids)  # multi-GPUs

    def forward(self, input_):
        if self.provider != 'cpu:':
            input_ = input_.cuda()
        with torch.no_grad():
            return self.model(input_)



# -- coding: utf-8 --
# @Time : 2022/8/29
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
from apstone import ONNXModel

if __name__ == '__main__':
    onnx_p = '../AI_power/pretrain_models/sr_lib/RealESRGAN_x4plus-dynamic.onnx'
    input_dynamic_shape = (1, 3, 768, 1080)
    # input_dynamic_shape = None

    # # cpu
    # ONNXModel(onnx_p, provider='cpu', debug=True, input_dynamic_shape=input_dynamic_shape).speed_test()

    # gpu
    ONNXModel(onnx_p, provider='gpu', debug=True, input_dynamic_shape=input_dynamic_shape).speed_test()

    # # trt
    # ONNXModel(onnx_p, provider='trt', debug=False, input_dynamic_shape=input_dynamic_shape).speed_test()
    #
    # # trt16
    # ONNXModel(onnx_p, provider='trt16', debug=False, input_dynamic_shape=input_dynamic_shape).speed_test()

    # # trt int8
    # ONNXModel(onnx_p, provider='trt8', debug=False, input_dynamic_shape=input_dynamic_shape).speed_test()



# -- coding: utf-8 --
# @Time : 2022/7/28
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
"""
mmdet model infer wrapper
"""
from cv2box import CVImage, try_import
import numpy as np
import cv2
from ..mmlab_wrapper.utils import _taylor, _gaussian_blur, _get_max_preds
transforms = try_import('torchvision.transforms', '')
from ...model_base import ModelBase


class KpDetectorBase(ModelBase):
    def __init__(self, model_info, provider):
        super().__init__(model_info, provider)
        self.dark_flag = None
        self.ratio = self.left = self.top = None
        if 'model_input_size' in model_info.keys():
            self.model_input_size = model_info['model_input_size']
        if 'kernel' in model_info.keys():
            self.kernel_size = model_info['kernel']
        else:
            self.kernel_size = 11

    def trans_pred(self, preds, ratio, left, top, maxvals, H):
        kp_results = []
        for index, kp in enumerate(preds[0]):
            x, y = preds[0][index]
            # 4 / ratio
            new_ratio = (self.model_input_size[1] // H) / ratio
            new_y = y * new_ratio + top
            new_x = x * new_ratio + left
            kp_results.append([new_x, new_y, float(maxvals[0][index][0])])
        # print(kp_results)
        return kp_results

    def post_process_default(self, heatmaps, ratio, left, top):
        # postprocess
        N, K, H, W = heatmaps.shape
        preds, maxvals = _get_max_preds(heatmaps)
        for n in range(N):
            for k in range(K):
                heatmap = heatmaps[n][k]
                px = int(preds[n][k][0])
                py = int(preds[n][k][1])
                if 1 <= px <= W - 2 and 1 <= py <= H - 2:
                    diff = np.array([
                        heatmap[py][px + 1] - heatmap[py][px - 1],
                        heatmap[py + 1][px] - heatmap[py - 1][px]
                    ])
                    preds[n][k] += np.sign(diff) * .25
        return self.trans_pred(preds, ratio, left, top, maxvals, H)

    def post_process_modelscope(self, heatmaps, ratio, left, top):
        N, K, H, W = heatmaps.shape
        _, maxvals = _get_max_preds(heatmaps)
        preds = []
        scores = []
        for i in range(K):
            heatmap = heatmaps[i, :, :]
            pt = np.where(heatmap == np.max(heatmap))
            scores.append(np.max(heatmap))
            x = pt[1][0]
            y = pt[0][0]

            if x >= 1 and x <= W - 2 and y >= 1 and y <= H - 2:
                x_diff = heatmap[y, x + 1] - heatmap[y, x - 1]
                y_diff = heatmap[y + 1, x] - heatmap[y - 1, x]
                x_sign = 0
                y_sign = 0
                if x_diff < 0:
                    x_sign = -1
                if x_diff > 0:
                    x_sign = 1
                if y_diff < 0:
                    y_sign = -1
                if y_diff > 0:
                    y_sign = 1
                x = x + x_sign * 0.25
                y = y + y_sign * 0.25

            preds.append([x, y])
        return self.trans_pred(preds, ratio, left, top, maxvals, H)

    def post_process_unbiased(self, heatmaps, ratio, left, top, kernel=11):
        # apply Gaussian distribution modulation.
        N, K, H, W = heatmaps.shape
        preds, maxvals = _get_max_preds(heatmaps)
        heatmaps = np.log(np.maximum(_gaussian_blur(heatmaps, kernel), 1e-10))
        for n in range(N):
            for k in range(K):
                preds[n][k] = _taylor(heatmaps[n][k], preds[n][k])
        return self.trans_pred(preds, ratio, left, top, maxvals, H)

    def preprocess(self, image_in_, bbox_, mirror=False):
        img_resize, self.ratio, self.left, self.top = CVImage(image_in_).crop_keep_ratio(bbox_, self.model_input_size)
        if mirror:
            img_resize = cv2.flip(img_resize, 1)
        if self.model_type == 'trt':
            transform = transforms.Compose([
                transforms.ToTensor(),
                transforms.Normalize((0.485, 0.456, 0.406), (0.229, 0.224, 0.225)),
            ])
            blob = dict(input=CVImage(img_resize).set_transform(transform).tensor().cuda())
        else:
            blob = CVImage(img_resize).innormal(mean=[123.675, 116.28, 103.53],
                                                std=[58.395, 57.1200, 57.375]).transpose(2, 0, 1)[np.newaxis, :]
        return blob

    # def preprocess_batch(self, image_in_batch_, bbox_batch_, mirror=False):
    #     img_resize, self.ratio, self.left, self.top = CVImage(image_in_).crop_keep_ratio(bbox_, self.model_input_size)
    #     if mirror:
    #         img_resize = cv2.flip(img_resize, 1)
    #     if self.model_type == 'trt':
    #         transform = transforms.Compose([
    #             transforms.ToTensor(),
    #             transforms.Normalize((0.485, 0.456, 0.406), (0.229, 0.224, 0.225)),
    #         ])
    #         blob = dict(input=CVImage(img_resize).set_transform(transform).tensor().cuda())
    #     else:
    #         blob = CVImage(img_resize).innormal(mean=[123.675, 116.28, 103.53],
    #                                             std=[58.395, 57.1200, 57.375], to_rgb=False).transpose(2, 0, 1)[np.newaxis, :]
    #     return blob

    def postprocess(self, model_results):
        if self.model_type == 'trt':
            heatmaps = model_results['output'].cpu().numpy()
        else:
            heatmaps = model_results[0]
        if self.dark_flag:
            kp_results = self.post_process_unbiased(heatmaps, self.ratio, self.left, self.top, kernel=self.kernel_size)
        else:
            kp_results = self.post_process_default(heatmaps, self.ratio, self.left, self.top)
        return kp_results

    def postprocess_mirror(self, model_results, model_mirror_results, flip_pairs):
        if self.model_type == 'trt':
            heatmaps = model_results['output'].cpu().numpy()
            heatmaps_mirror = model_mirror_results['output'].cpu().numpy()
        else:
            heatmaps = model_results[0]
            heatmaps_mirror = model_mirror_results[0]

        heatmaps_mirror_back = heatmaps_mirror.copy()
        # # Swap left-right parts
        for left_id, right_id in flip_pairs:
            heatmaps_mirror_back[:, left_id, ...] = heatmaps_mirror[:, right_id, ...]
            heatmaps_mirror_back[:, right_id, ...] = heatmaps_mirror[:, left_id, ...]
        heatmaps_mirror_back = heatmaps_mirror_back[..., ::-1]
        heatmaps = (heatmaps + heatmaps_mirror_back) * 0.5

        if self.model_type.find('dark') > 0:
            kp_results = self.post_process_default(heatmaps, self.ratio, self.left, self.top)
        else:
            kp_results = self.post_process_unbiased(heatmaps, self.ratio, self.left, self.top)
        return kp_results

    def show(self, image_in_, results_):
        img_origin = CVImage(image_in_).bgr.copy()
        img_show = CVImage(img_origin).draw_landmarks(results_)
        CVImage(img_show).show(0, "human kp result")



# -- coding: utf-8 --
# @Time : 2022/11/7
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone

from modelscope.pipelines import pipeline
from modelscope.utils.constant import Tasks
from cv2box import CVImage

model_id = 'damo/cv_hrnetv2w32_body-2d-keypoints_image'
body_2d_keypoints = pipeline(Tasks.body_2d_keypoints, model=model_id)
output = body_2d_keypoints('resource/for_pose/t_pose_1080p.jpeg')

# the output contains poses, scores and boxes
print(output)
CVImage(CVImage('resource/for_pose/t_pose_1080p.jpeg').draw_landmarks(output['keypoints'][0])).show()



# -- coding: utf-8 --
# @Time : 2022/2/14
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
import tensorflow as tf

# 通过输出的结点名，将与这个结点有关的图及权重保存下来
constant_graph = tf.graph_util.convert_variables_to_constants(sess, sess.graph_def,
                                                              output_node_names=[input_name.split(':')[0]])
# 写入序列化的 PB 文件，获取带权重的图
with tf.gfile.FastGFile('model.pb', mode='wb') as f:
    f.write(constant_graph.SerializeToString())

# # 查看input output name   https://netron.app/

# # 利用tf2onnx将pb转为onnx  https://github.com/onnx/tensorflow-onnx
# pip install tf2onnx
# python -m tf2onnx.convert --graphdef model.pb --output model.onnx --inputs 'Placeholder:0' --outputs 'select:0'



# -- coding: utf-8 --
# @Time : 2023/5/6
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone

class LLM:
    def __init__(self, model_info, load_in_8bit, device_map):
        self.model_path = model_info['model_path']
        if 'self.config' not in locals():
            self.config = model_info['config'].from_pretrained(self.model_path, return_unused_kwargs=True, trust_remote_code=True)[0]
        self.model = model_info['model'].from_pretrained(self.model_path, trust_remote_code=True,
                                                         load_in_8bit=load_in_8bit, device_map=device_map())
        self.tokenizer = model_info['tokenizer'].from_pretrained(self.model_path, trust_remote_code=True)

    def generate(self):
        pass


# -- coding: utf-8 --
# @Time : 2022/7/29
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone

from pathlib import Path
from .wrappers.onnx_wrapper import ONNXModel, OnnxModelPickable


class ModelBase:
    def __init__(self, model_info, provider):
        self.model_path = model_info['model_path']

        if 'input_dynamic_shape' in model_info.keys():
            self.input_dynamic_shape = model_info['input_dynamic_shape']
        else:
            self.input_dynamic_shape = None

        if 'picklable' in model_info.keys():
            picklable = model_info['picklable']
        else:
            picklable = False

        # init model
        if Path(self.model_path).suffix == '.engine':
            if 'trt_wrapper_self' in model_info.keys():
                from .wrappers.trt_wrapper import TRTWrapperSelf
                TRTWrapper = TRTWrapperSelf
            else:
                from .wrappers.trt_wrapper import TRTWrapper
            self.model_type = 'trt'
            self.model = TRTWrapper(self.model_path)
        elif Path(self.model_path).suffix == '.tjm':
            from .wrappers.tjm_wrapper import TJMWrapper
            self.model_type = 'tjm'
            self.model = TJMWrapper(self.model_path, provider=provider)
        elif Path(self.model_path).suffix in ['.onnx', '.bin']:
            self.model_type = 'onnx'
            if not picklable:
                if 'encrypt' in model_info.keys():
                    from cv2box.utils.encrypt import CVEncrypt
                    self.model_path = CVEncrypt(model_info['encrypt']).load_encrypt_file(self.model_path)
                self.model = ONNXModel(self.model_path, provider=provider,
                                              input_dynamic_shape=self.input_dynamic_shape)
            else:
                self.model = OnnxModelPickable(self.model_path, provider=provider, )

        else:
            raise 'check model suffix , support engine/tjm/onnx/bin now.'



# -- coding: utf-8 --
# @Time : 2022/7/28
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
"""
mmdet model infer wrapper
"""
from cv2box import CVImage, try_import
import numpy as np
import cv2
transforms = try_import('torchvision.transforms', '')
from ...model_base import ModelBase


class BboxDetectorBase(ModelBase):
    def __init__(self, model_info, provider):
        super().__init__(model_info, provider)
        self.ratio = self.pad_w = self.pad_h = None
        self.model_input_size = model_info['model_input_size']

    def preprocess(self, image_in_):
        img_resize, self.ratio, self.pad_w, self.pad_h = CVImage(image_in_).resize_keep_ratio(self.model_input_size,
                                                                                              pad_value=(
                                                                                                  114.0, 114.0, 114.0))
        if self.model_type == 'trt':
            transform = transforms.Compose([
                transforms.ToTensor(),
            ])
            blob = dict(input=CVImage(img_resize.astype(np.float32)).set_transform(transform).tensor().cuda())
        else:
            blob = img_resize.astype(np.float32).transpose(2, 0, 1)[np.newaxis, :]
        return blob

    def postprocess(self, model_results, threshold, label=0, max_bbox_num=99):
        if self.model_type == 'trt':
            dets = model_results['dets'].cpu().numpy()
            labels = model_results['labels'].cpu().numpy()
        else:
            dets = model_results[0]
            labels = model_results[1]
        results = []
        for index, bbox in enumerate(dets[0]):
            if labels[0][index] == label and bbox[4] > threshold:
                results.append(
                    np.concatenate((CVImage(None).recover_from_resize([bbox[:2]], self.ratio, self.pad_w, self.pad_h)[0],
                                    CVImage(None).recover_from_resize([bbox[2:4]], self.ratio, self.pad_w, self.pad_h)[
                                        0])).astype(int))
        return results[:max_bbox_num]

    def show(self, image_in_, results_):
        img_origin = CVImage(image_in_).bgr.copy()
        for bbox_ in results_:
            cv2.rectangle(img_origin, (bbox_[0], bbox_[1]), (bbox_[2], bbox_[3]), (0, 255, 0), 4)
        CVImage(img_origin).show(0, "bbox result")
        return img_origin



# -- coding: utf-8 --
# @Time : 2022/7/27
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
"""
pip install -U onnx \
&& pip install -U sio4onnx
"""
from sio4onnx import io_change

estimated_graph = io_change(
    input_onnx_file_path="pretrain_models/hand_lib/hand_detector_yolox/yolox_100DOH_epoch90.onnx",
    output_onnx_file_path="pretrain_models/hand_lib/hand_detector_yolox/yolox_100DOH_epoch90_fix_input.onnx",
    input_names=[
        "input",
    ],
    input_shapes=[
        [1, 3, 640, 640],
    ],
    output_names=[
        "dets",
        "labels",
    ],
    output_shapes=[
        [1, "num_dets", 5],
        [1, "num_dets"],
    ],
)



from cv2box import try_import
torch = try_import('torch', 'torch2any need torch !')


def torch2jit(model_, input_):
    traced_gpu = torch.jit.trace(model_, input_)
    torch.jit.save(traced_gpu, "gpu.tjm")


def torch2onnx(model_, input_, output_name="./test.onnx"):
    input_names = ["input_1"]
    output_names = ["output_1"]
    opset_version = 13
    dynamic_axes = None
    # dynamic_axes = {'input_1': [0, 2, 3], 'output_1': [0, 1]}
    torch.onnx.export(model_, input_, output_name, verbose=True, opset_version=opset_version,
                      input_names=input_names, output_names=output_names,
                      dynamic_axes=dynamic_axes, do_constant_folding=True)
    raise 'convert done !'



# -- coding: utf-8 --
# @Time : 2021/11/29
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone


import onnxruntime
import numpy as np
from cv2box import MyFpsCounter
import os
from pathlib import Path
import re


def get_output_info(onnx_session):
    output_name = []
    output_shape = []
    for node in onnx_session.get_outputs():
        output_name.append(node.name)
        output_shape.append(node.shape)
    return output_name, output_shape


def get_input_info(onnx_session):
    input_name = []
    input_shape = []
    for node in onnx_session.get_inputs():
        input_name.append(node.name)
        input_shape.append(node.shape)
    return input_name, input_shape


def get_input_feed(input_name, image_tensor):
    """
    Args:
        input_name:
        image_tensor: [image tensor, ...]
    Returns:
    """
    input_feed = {}
    for index, name in enumerate(input_name):
        input_feed[name] = image_tensor[index]
    return input_feed


class ONNXModel:
    def __init__(self, onnx_path, provider='gpu', debug=False, input_dynamic_shape=None):
        self.provider = provider
        try:
            onnx_name = Path(onnx_path).stem
            trt_cache_path = './cache/trt/' + onnx_name
        except TypeError:
            pass

        if self.provider == 'gpu':
            self.providers = (
                "CUDAExecutionProvider",
                {'device_id': 0, }
            )
        elif self.provider == 'trt':
            os.makedirs(trt_cache_path, exist_ok=True)
            self.providers = (
                'TensorrtExecutionProvider',
                {'trt_engine_cache_enable': True, 'trt_engine_cache_path': trt_cache_path, 'trt_fp16_enable': False, }
            )
        elif self.provider == 'trt16':
            os.makedirs(trt_cache_path, exist_ok=True)
            self.providers = (
                'TensorrtExecutionProvider',
                {'trt_engine_cache_enable': True, 'trt_engine_cache_path': trt_cache_path, 'trt_fp16_enable': True,
                 'trt_dla_enable': False, }
            )
        elif self.provider == 'trt8':
            os.makedirs(trt_cache_path, exist_ok=True)
            self.providers = (
                'TensorrtExecutionProvider',
                {'trt_engine_cache_enable': True, 'trt_int8_enable': True, }
            )
        else:
            self.providers = "CPUExecutionProvider"

        # onnxruntime.set_default_logger_severity(3)
        session_options = onnxruntime.SessionOptions()
        session_options.log_severity_level = 3

        # When env change leads to trt cache load fail, auto generate new cache
        try:
            self.onnx_session = onnxruntime.InferenceSession(onnx_path, session_options, providers=[self.providers])
        except Exception as e:
            if type(e.args[0])==str and 'TensorRT EP could not deserialize engine from cache' in e.args[0]:
                res = re.match('.*TensorRT EP could not deserialize engine from cache: (.*)', e.args[0])
                os.remove(res.group(1))
                print('waiting generate new model...')
                self.onnx_session = onnxruntime.InferenceSession(onnx_path, session_options, providers=[self.providers])
            else:
                raise e

        # sessionOptions.intra_op_num_threads = 3
        self.input_name, self.input_shape = get_input_info(self.onnx_session)
        self.output_name, self.output_shape = get_output_info(self.onnx_session)

        self.input_dynamic_shape = input_dynamic_shape

        if self.input_dynamic_shape is not None:
            self.input_dynamic_shape = self.input_dynamic_shape if isinstance(self.input_dynamic_shape, list) else [
                self.input_dynamic_shape]

        if debug:
            print('onnx version: {}'.format(onnxruntime.__version__))
            print("input_name:{}, shape:{}".format(self.input_name, self.input_shape))
            print("output_name:{}, shape:{}".format(self.output_name, self.output_shape))

        self.warm_up()

    def warm_up(self):
        if not self.input_dynamic_shape:
            try:
                self.forward([np.random.rand(*self.input_shape[i]).astype(np.float32)
                              for i in range(len(self.input_shape))])
            except TypeError:
                print('Model may be dynamic, plz name the \'input_dynamic_shape\' !')
        else:
            self.forward([np.random.rand(*self.input_dynamic_shape[i]).astype(np.float32)
                          for i in range(len(self.input_shape))])
        print('Model warm up done !')

    def speed_test(self):
        if not self.input_dynamic_shape:
            input_tensor = [np.random.rand(*self.input_shape[i]).astype(np.float32)
                            for i in range(len(self.input_shape))]
        else:
            input_tensor = [np.random.rand(*self.input_dynamic_shape[i]).astype(np.float32)
                            for i in range(len(self.input_shape))]

        with MyFpsCounter('[{}] onnx 10 times'.format(self.provider)) as mfc:
            for i in range(10):
                _ = self.forward(input_tensor)

    def forward(self, image_tensor_in, trans=False):
        """
        Args:
            image_tensor_in: image_tensor [image_tensor] [image_tensor_1, image_tensor_2]
            trans: apply trans for image_tensor or first image_tensor(list)
        Returns:
            model output
        """
        if not isinstance(image_tensor_in, list) or len(image_tensor_in) == 1:
            image_tensor_in = image_tensor_in[0] if isinstance(image_tensor_in, list) else image_tensor_in
            if trans:
                image_tensor_in = image_tensor_in.transpose(2, 0, 1)[np.newaxis, :]
            image_tensor_in = [np.ascontiguousarray(image_tensor_in)]
        else:
            # for multi input, only trans first tensor
            if trans:
                image_tensor_in[0] = image_tensor_in[0].transpose(2, 0, 1)[np.newaxis, :]
            image_tensor_in = [np.ascontiguousarray(image_tensor) for image_tensor in image_tensor_in]

        input_feed = get_input_feed(self.input_name, image_tensor_in)
        return self.onnx_session.run(self.output_name, input_feed=input_feed)

    def batch_forward(self, bach_image_tensor, trans=False):
        if trans:
            bach_image_tensor = bach_image_tensor.transpose(0, 3, 1, 2)
        input_feed = get_input_feed(self.input_name, bach_image_tensor)
        return self.onnx_session.run(self.output_name, input_feed=input_feed)

    def binding_forward(self, binding):
        """
        ref io_binding https://onnxruntime.ai/docs/api/python/api_summary.html
        do io_binding outside this func and pass it in
        example(cuda in&out):
            import torch
            binding = self.model.onnx_session.io_binding()
            image_tensor_in = image_tensor_in.contiguous()
            binding.bind_input(
                name='X',
                device_type='cuda',
                device_id=0,
                element_type=np.float32,
                shape=tuple(image_tensor_in.shape),
                buffer_ptr=image_tensor_in.data_ptr(),)

            Y_tensor = torch.empty(Y_shape, dtype=torch.float32, device='cuda:0').contiguous()
            binding.bind_output(
                name='Y',
                device_type='cuda',
                device_id=0,
                element_type=np.float32,
                shape=tuple(Y_tensor.shape),
                buffer_ptr=Y_tensor.data_ptr(),)
        """
        self.onnx_session.run_with_iobinding(binding)



# -- coding: utf-8 --
# @Time : 2022/6/28
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone

import numpy as np
import cv2
import numba


@numba.jit(nopython=True)
def _taylor_numba(heatmap, coord):
    """Distribution aware coordinate decoding method.

    Note:
        - heatmap height: H
        - heatmap width: W

    Args:
        heatmap (np.ndarray[H, W]): Heatmap of a particular joint type.
        coord (np.ndarray[2,]): Coordinates of the predicted keypoints.

    Returns:
        np.ndarray[2,]: Updated coordinates.
    """
    offset_flag = False
    offset = np.zeros((2, 1))
    H, W = heatmap.shape[:2]
    px, py = int(coord[0]), int(coord[1])
    if 1 < px < W - 2 and 1 < py < H - 2:
        dx = 0.5 * (heatmap[py][px + 1] - heatmap[py][px - 1])
        dy = 0.5 * (heatmap[py + 1][px] - heatmap[py - 1][px])
        dxx = 0.25 * (
                heatmap[py][px + 2] - 2 * heatmap[py][px] + heatmap[py][px - 2])
        dxy = 0.25 * (
                heatmap[py + 1][px + 1] - heatmap[py - 1][px + 1] -
                heatmap[py + 1][px - 1] + heatmap[py - 1][px - 1])
        dyy = 0.25 * (
                heatmap[py + 2 * 1][px] - 2 * heatmap[py][px] +
                heatmap[py - 2 * 1][px])
        derivative = np.array([[dx], [dy]])
        hessian = np.array([[dxx, dxy], [dxy, dyy]])
        if dxx * dyy - dxy ** 2 != 0:
            hessianinv = np.linalg.inv(hessian)
            offset = -hessianinv @ derivative
            offset_flag = True

    return coord, offset_flag, offset


def _taylor(heatmap, coord):
    """Distribution aware coordinate decoding method.

    Note:
        - heatmap height: H
        - heatmap width: W

    Args:
        heatmap (np.ndarray[H, W]): Heatmap of a particular joint type.
        coord (np.ndarray[2,]): Coordinates of the predicted keypoints.

    Returns:
        np.ndarray[2,]: Updated coordinates.
    """
    H, W = heatmap.shape[:2]
    px, py = int(coord[0]), int(coord[1])
    if 1 < px < W - 2 and 1 < py < H - 2:
        dx = 0.5 * (heatmap[py][px + 1] - heatmap[py][px - 1])
        dy = 0.5 * (heatmap[py + 1][px] - heatmap[py - 1][px])
        dxx = 0.25 * (
                heatmap[py][px + 2] - 2 * heatmap[py][px] + heatmap[py][px - 2])
        dxy = 0.25 * (
                heatmap[py + 1][px + 1] - heatmap[py - 1][px + 1] -
                heatmap[py + 1][px - 1] + heatmap[py - 1][px - 1])
        dyy = 0.25 * (
                heatmap[py + 2 * 1][px] - 2 * heatmap[py][px] +
                heatmap[py - 2 * 1][px])
        derivative = np.array([[dx], [dy]])
        hessian = np.array([[dxx, dxy], [dxy, dyy]])
        if dxx * dyy - dxy ** 2 != 0:
            hessianinv = np.linalg.inv(hessian)
            offset = -hessianinv @ derivative
            offset = np.squeeze(np.array(offset.T), axis=0)
            coord += offset
    return coord


def _gaussian_blur(heatmaps, kernel=11):
    """Modulate heatmap distribution with Gaussian.
     sigma = 0.3*((kernel_size-1)*0.5-1)+0.8
     sigma~=3 if k=17
     sigma=2 if k=11;
     sigma~=1.5 if k=7;
     sigma~=1 if k=3;

    Note:
        - batch_size: N
        - num_keypoints: K
        - heatmap height: H
        - heatmap width: W

    Args:
        heatmaps (np.ndarray[N, K, H, W]): model predicted heatmaps.
        kernel (int): Gaussian kernel size (K) for modulation, which should
            match the heatmap gaussian sigma when training.
            K=17 for sigma=3 and k=11 for sigma=2.

    Returns:
        np.ndarray ([N, K, H, W]): Modulated heatmap distribution.
    """
    assert kernel % 2 == 1

    border = (kernel - 1) // 2
    batch_size = heatmaps.shape[0]
    num_joints = heatmaps.shape[1]
    height = heatmaps.shape[2]
    width = heatmaps.shape[3]
    for i in range(batch_size):
        for j in range(num_joints):
            origin_max = np.max(heatmaps[i, j])
            dr = np.zeros((height + 2 * border, width + 2 * border),
                          dtype=np.float32)
            dr[border:-border, border:-border] = heatmaps[i, j].copy()
            dr = cv2.GaussianBlur(dr, (kernel, kernel), 0)
            heatmaps[i, j] = dr[border:-border, border:-border].copy()
            heatmaps[i, j] *= origin_max / np.max(heatmaps[i, j])
    return heatmaps


def _get_max_preds(heatmaps):
    """Get keypoint predictions from score maps.

    Note:
        batch_size: N
        num_keypoints: K
        heatmap height: H
        heatmap width: W

    Args:
        heatmaps (np.ndarray[N, K, H, W]): model predicted heatmaps.

    Returns:
        tuple: A tuple containing aggregated results.

        - preds (np.ndarray[N, K, 2]): Predicted keypoint location.
        - maxvals (np.ndarray[N, K, 1]): Scores (confidence) of the keypoints.
    """
    assert isinstance(heatmaps,
                      np.ndarray), ('heatmaps should be numpy.ndarray')
    assert heatmaps.ndim == 4, 'batch_images should be 4-ndim'

    N, K, _, W = heatmaps.shape
    heatmaps_reshaped = heatmaps.reshape((N, K, -1))
    idx = np.argmax(heatmaps_reshaped, 2).reshape((N, K, 1))
    maxvals = np.amax(heatmaps_reshaped, 2).reshape((N, K, 1))

    preds = np.tile(idx, (1, 1, 2)).astype(np.float32)
    preds[:, :, 0] = preds[:, :, 0] % W
    preds[:, :, 1] = preds[:, :, 1] // W

    preds = np.where(np.tile(maxvals, (1, 1, 2)) > 0.0, preds, -1)
    return preds, maxvals



#!/usr/bin/env python
# coding: utf-8

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name='apstone',  # 项目的名称
    version='0.0.5',  # 项目版本
    author='ykk648',  # 项目作者
    author_email='ykk648@gmail.com',  # 作者email
    url='https://github.com/ykk648/apstone',  # 项目代码仓库
    project_urls={
        "Bug Tracker": "https://github.com/ykk648/apstone/issues",
    },
    description='ai_power base stone',  # 项目描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    # package_dir={"": "src"},
    packages=setuptools.find_packages(),
    extras_require={"light": [],
                    "full": ['apscheduler', 'numexpr', 'h5py', 'yaml']},
    install_requires=['cv2box'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)



# -- coding: utf-8 --
# @Time : 2022/6/24
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
import torch

torch_model_path = "/hmr_ava.pt"
model = torch.load(torch_model_path)['model']
print(model.keys())
# torch.save(model, 'model.pt')

new_model = model.copy()
for key in model.keys():
    # print(key)
    new_model.pop(key)
    new_model[key.replace('smplx_', '')] = model[key]

print(new_model.keys())

torch.save(new_model, '')
